import os, sys, traceback, linecache
from mext.tz import local_tz
from pasteob.debug.utils import *

__all__ = ['CollectedException']


class Traceback(object):
    def __init__(self, exc_info):
        assert exc_info
        self.exc_type, self.exc, tb = exc_info
        self.timestamp = local_tz.now()
        self.frames = map(TracebackFrame.from_tb, iter_tb(tb))

        self.extra_data = {}
        if self.exc_type is ImportError:
            self.extra_data['sys.path'] = [sys.path]

    @classmethod
    def collect(cls):
        return cls(sys.exc_info())


class TracebackFrame(object):
    @classmethod
    def from_tb(cls, tb):
        return cls(frame=tb.tb_frame, line_no=tb.tb_lineno)

    def __init__(self, frame, line_no):
        co = frame.f_code
        globals = frame.f_globals
        self.globals = globals
        self.locals = frame.f_locals

        if os.name == 'nt':
            fs_charset = 'mbcs'
        else:
            fs_charset = 'utf8'
        self.module_name = maybe_unicode_safe(globals.get('__name__'), fs_charset)
        self.filename = maybe_unicode_safe(co.co_filename, fs_charset)
        self.line_no = line_no
        self.func_name = maybe_unicode_safe(co.co_name)

    _source_encoding = None

    def get_source_line(self, context=0):
        if not self.filename or not self.line_no:
            return None
        if not self._source_encoding:
            lines = [linecache.getline(self.filename, i) for i in (1,2)]
            self._source_encoding = detect_source_encoding(lines) or 'ascii'
        lines = []
        for lineno in range(self.line_no-context, self.line_no+context+1):
            lines.append(linecache.getline(self.filename, lineno))
        return ''.join(lines).decode(self._source_encoding)





def iter_tb(tb):
    while tb is not None:
        yield tb
        tb = tb.tb_next





########################################
########################################

# based on weberror.util.source_encoding
# for original copyright, see http://tinyurl.com/mxe8dz
import re, codecs

_rx_magic_comment = re.compile(
    r'[ \t\f]* \# .* coding[=:][ \t]*([-\w.]+)',
    re.VERBOSE
)

def detect_source_encoding((line1, line2)):
    """
        Deduce the encoding of a source file from magic comment.
    """
    if line1.startswith(codecs.BOM_UTF8):
        return 'UTF-8-sig'
    m = _rx_magic_comment.match(line1) or _rx_magic_comment.match(line2)
    if m:
        return m.group(1)
