import sys
from pasteob import *
from mext.htmlgen import *

from pygments.formatters import HtmlFormatter
from pygments import highlight as pygments_highlight
from pygments.lexers import PythonLexer
pygments_css = HtmlFormatter().get_style_defs('.highlight')

__all__ = ['show_module_source_app']


@webob_wrap
@staticmethod
def show_module_source_app(req):
    module_name = req.path_info.lstrip('/')
    module = sys.modules.get(module_name)
    if module is not None:
        if not hasattr(module, '__file__'):
            return Response('<builtin module %s>' % module_name, content_type='text/plain')
        filename, source = get_source(module)
        html = HTML
        html += TITLE[module_name, ' - ', filename]
        html += STYLE[pygments_css]
        html += raw_html(highlight_src(source, linenos=True))
        html = str(HtmlDocument(html))
        return Response(html)



def highlight_src(
    code,
    linenos=False,
    lineanchors='code',
    cssclass='highlight'
):
    lexer = PythonLexer()
    formatter = HtmlFormatter(
        linenos=linenos,
        lineanchors=lineanchors,
        cssclass=cssclass
    )
    return pygments_highlight(code, lexer, formatter)

def get_source(module):
    filename = module.__file__
    if filename[-4:] in ('.pyc', '.pyo'):
        filename = filename[:-1]
    elif filename.endswith('$py.class'):
        filename = '%s.py' % filename[:-9]
    source = open(filename, 'rb').read()
    return filename, source



